(in-package "USER")

(defun recomp ()
  
  (compile-file "saga-simplify.lisp")
  (compile-file "saga.lisp")
  (compile-file "saga-display.lisp")
  (compile-file "saga-trans-tancs.lisp")
  (compile-file "saga-bench-tancs.lisp")
  (compile-file "saga-debug.lisp")
  
  (load "saga.fasl")
  (load "saga-trans-tancs.fasl")
  (load "saga-bench-tancs.fasl")
  (load "saga-debug.fasl")
  (load "saga-display.fasl"))

(defvar *begin*)
(defvar *maxtime*)
(defvar *listfile*)
(defvar *protocol-file*)
(defvar *verbose*)

(setf *maxtime* 60000)
(setf *listfile* "qbf-comp-list")	; or psat-comp-list
(setf *protocol-file* "qbf-details-subset.txt")
(setf *verbose* t)

(setf *early-flatten* t)

(defun fulltbench ()
  
  (dolist (sim '(t nil))
    (dolist (sequence '((atom and eq all ex or) (atom and eq all or ex)))
      (dolist (sb '(t nil))
	(dolist (moms '(t nil))
	  (dolist (imoms '(t nil))
	    (dolist (block '(t nil))
	      (dolist (bj '(t nil))
		(setf *sequence* sequence
		      *simplify* sim
		      *blocking* block
		      *moms* moms
		      *imoms* imoms
		      *backjump* bj
		      *sem-branch* sb)
	      
	      (unless (or
		       (and (not *sem-branch*) *moms*) ; moms without sb
		       (and (not *moms*) *imoms*)) ; imoms witout moms
		
		(let ((number (tbench)))
		  
		  (with-open-file
		      (outstream *protocol-file* 
		       :direction :output
		       :if-exists :append
		       :if-does-not-exist :create)
		    (format outstream "Result is ~a~%" number)
		    
		    (format outstream 
			    "Parameters were seq ~a simpl ~a sb ~a moms ~a 
imoms ~a blocking ~a bj ~a~%" sequence sim sb moms imoms block bj))))))))))))

(defun tbench ()
  (excl:advise satisfy :before watchdog nil 
	       (when (> (- (get-internal-run-time) *begin*) (* *maxtime*  (/ internal-time-units-per-second 100))) 
		 (throw 'timeout nil)))
  
  (let  ((total-number 0))
  (with-open-file (in-stream *listfile* :direction :input)
    (do ((nextline (read-line in-stream nil nil)
		   (read-line in-stream nil nil)))
	((null nextline) nil)
      (let* ((tancs-formula nextline)
	     (saga-formula (transfile tancs-formula))
	     satisfiable begin end time timeout out-of-mem result)
		
	(excl:gc t)			; global gc
	(setf begin (get-internal-run-time))
	(setf *begin* begin)
	
	(handler-case 
	    (catch 'timeout
	      (setf satisfiable (construct-tableau saga-formula)))
	  (storage-condition () (setf out-of-mem t)))
	
	(setf end (get-internal-run-time))
	(setf time (/ (- end begin) (/ internal-time-units-per-second 100)))
	
	(if (> time *maxtime*)
	    (setf timeout t)
	  (setf timeout nil))
	(setf result 
	  (cond (timeout "T")
		(out-of-mem "M")
		(satisfiable "S")
		(t "U")))
	
	(when *verbose*
	  (with-open-file
	      (outstream *protocol-file* 
	       :direction :output
	       :if-exists :append
	       :if-does-not-exist :create)
	    (format outstream "i ~a ~a ~a~%" tancs-formula time result)))
	
	(unless (or timeout out-of-mem)
	  (incf total-number)))))
  total-number))










