;;; File saga-trans-tancs.lisp
;;; translates tancs formulae to saga-syntax

(in-package "USER")

(setf *top* '(:top)
      *bottom* '(:not (:top)))

(defvar *next-var*)

(defun get-next-var ()
  (list '? (incf *next-var*)))
    

(defun transfile (in-file)
  (setf *next-var* 0)
  (let ((first-var (get-next-var))
	formula kind)
    (setf formula (list (list 'a first-var)))
    (with-open-file (in-stream in-file :direction :input)
      (do ((nextline (read-line in-stream nil nil)
		     (read-line in-stream nil nil)))
	  ((null nextline) nil)
	(with-input-from-string (input nextline)
	  (let ((next-object (read input nil nil))
		new-form new-var)
	    (unless (or (null next-object) (eq '% next-object))
	      ;; skip empty lines and comment
					;	      (print next-object)
	      (case next-object
		((hypo conj axiom) (setf kind next-object))
		(otherwise
		 (setf new-form
		   (case kind
		     ('hypo (trans next-object first-var))
		     ('conj (list ':not (trans next-object first-var)))
		     ('axiom (setf new-var (get-next-var))
			     (list ':all 
				   (list new-var)
				   (list ':eq new-var new-var)
				   (trans next-object new-var)))))
		 
		 (push new-form formula))))))))
    (cons ':and formula)))
	  
(defun trans (form var)
  (if (listp form)
      (case (first form)
	('box
	 (let ((new-var (get-next-var)))
	   (case (second form)
	     ('r1
	      (list ':all (list new-var) (list 'r var new-var) (trans (third form) new-var)))
	     ('r1-
	      (list ':all (list new-var) (list 'r new-var var) (trans (third form) new-var)))
	     (otherwise
	      (transerror form)))))
	('pos
	 (let ((new-var (get-next-var)))
	   (case (second form)
	     ('r1
	      (list ':ex (list new-var) (list 'r var new-var) (trans (third form) new-var)))
	     ('r1-
	      (list ':ex (list new-var) (list 'r new-var var) (trans (third form) new-var)))
	     (otherwise
	      (transerror form)))))
	('~
	 (list ':not (trans (second form) var)))
	
	(otherwise
	 (case (second form)
	   ('&
	    (list ':and (trans (first form) var) (trans (rest (rest form)) var)))
	   ('+
	    (list ':or (trans (first form) var) (trans (rest (rest form)) var)))
	   (otherwise
	    (if (rest form)
		(transerror form)
	      ;; if form contains only one element, translate it
	      (trans (first form) var))))))
    
    ;; not listp
    (case form
      ('true *top*)
      ('false *bottom*)
      (nil
       (transerror form))
      (otherwise
       (list form var)))))


(defun transerror (term)
  (format t "trans error: ~s~%" term))

